'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
//--------------------------------------------------
// Properties
//--------------------------------------------------
var count = 0;

//--------------------------------------------------
// Functions
//--------------------------------------------------
/**
 * Make sure all props are strings.
 *
 * @param {String} [props] -
 */
function checkProps() {
  for (var _len = arguments.length, props = Array(_len), _key = 0; _key < _len; _key++) {
    props[_key] = arguments[_key];
  }

  props.forEach(function (prop) {
    if (typeof prop !== 'string') {
      throw new Error('Enum property \'' + prop + '\' must be a string.');
    }
  });
}

//--------------------------------------------------
// Factories
//--------------------------------------------------
/**
 * Create a new enum.
 * @public
 *
 * @param {String} [props] Keys to add to the enum.
 * @returns {Enum} Returns an enum wrapper.
 */
function enumFactory() {
  for (var _len2 = arguments.length, props = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    props[_key2] = arguments[_key2];
  }

  checkProps.apply(undefined, props);

  var Enum = function Enum(item) {
    Object.defineProperty(this, item, {
      enumerable: true,
      value: Symbol(item)
    });

    Object.defineProperty(this, 'toString', {
      enumerable: false,
      value: function value() {
        return item;
      }
    });
  };

  Object.defineProperty(Enum, 'name', {
    value: 'ENUM_' + count++
  });

  props.forEach(function (prop) {
    Object.defineProperty(Enum, prop, {
      enumerable: true,
      value: new Enum(prop)
    });
  });

  Object.defineProperty(Enum, 'add', {
    value: function value(prop) {
      Object.defineProperty(Enum, prop, {
        enumerable: true,
        value: new Enum(prop)
      });
    }
  });

  return Enum;
}

//--------------------------------------------------
// Exports
//--------------------------------------------------
// Factories
exports.default = enumFactory;